"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var buildFullPath = require('axios/lib/core/buildFullPath');
var createError = require('axios/lib/core/createError');
var settle = require('axios/lib/core/settle');
var buildURL = require('axios/lib/helpers/buildURL');
var validResponseTypes = ['arraybuffer', 'blob', 'json', 'text'];
var titaniumAdapter = function (config) {
    var _a = config.url, url = _a === void 0 ? '' : _a, _b = config.method, method = _b === void 0 ? 'get' : _b, _c = config.baseURL, baseURL = _c === void 0 ? '' : _c, _d = config.headers, headers = _d === void 0 ? {} : _d, params = config.params, paramsSerializer = config.paramsSerializer, requestData = config.data, timeout = config.timeout, auth = config.auth, responseType = config.responseType, onUploadProgress = config.onUploadProgress, onDownloadProgress = config.onDownloadProgress, cancelToken = config.cancelToken;
    // check responseType is supported
    if (responseType && validResponseTypes.indexOf(responseType) === -1) {
        throw new Error("Invalid response type " + responseType + ". Valid response types are " + validResponseTypes.join(', ') + ".");
    }
    var client = Ti.Network.createHTTPClient(typeof timeout !== 'undefined' ? { timeout: timeout } : {});
    if (onDownloadProgress) {
        var responseContentLength_1;
        client.onreadystatechange = function () {
            if (client.readyState === client.HEADERS_RECEIVED) {
                var contentLength = Number(client.getResponseHeader('Content-Length'));
                if (contentLength) {
                    responseContentLength_1 = contentLength;
                }
            }
        };
        client.ondatastream = function (_a) {
            var progress = _a.progress;
            var lengthComputable = progress !== Ti.Network.PROGRESS_UNKNOWN && typeof responseContentLength_1 !== 'undefined';
            onDownloadProgress({
                lengthComputable: lengthComputable,
                loaded: lengthComputable ? responseContentLength_1 * progress : 0,
                total: lengthComputable ? responseContentLength_1 : 0,
            });
        };
    }
    if (onUploadProgress) {
        var length_1 = requestData.length;
        client.onsendstream = function (_a) {
            var progress = _a.progress;
            var lengthComputable = progress !== Ti.Network.PROGRESS_UNKNOWN && typeof length_1 === 'number';
            onUploadProgress({
                lengthComputable: lengthComputable,
                loaded: lengthComputable ? length_1 * progress : 0,
                total: lengthComputable ? length_1 : 0,
            });
        };
    }
    return new Promise(function (resolve, reject) {
        client.onload = function () {
            var data;
            if (responseType === 'arraybuffer') {
                var buffer = Ti.createBuffer({ length: client.responseData.length });
                var blobStream = Ti.Stream.createStream({
                    source: client.responseData,
                    mode: Ti.Stream.MODE_READ,
                });
                blobStream.read(buffer);
                data = buffer;
            }
            else if (responseType === 'blob') {
                data = client.responseData;
            }
            else {
                data = client.responseText;
            }
            var response = {
                status: client.status,
                statusText: client.statusText,
                headers: client.allResponseHeaders,
                config: config,
                request: client,
                data: data,
            };
            settle(resolve, reject, response);
        };
        client.onerror = function (_a) {
            var error = _a.error, code = _a.code;
            var codeStr = typeof code !== 'undefined' ? code.toString() : undefined;
            reject(createError("Network error \"" + error + "\"", config, codeStr, client));
        };
        // handle cancelToken
        if (cancelToken) {
            cancelToken.promise.then(function (cancel) {
                client.abort();
                reject(cancel);
            });
        }
        // open request
        var fullPath = buildFullPath(baseURL, url);
        client.open(method.toUpperCase(), buildURL(fullPath, params, paramsSerializer), true);
        // set request headers
        if (auth) {
            headers.Authorization = "Basic " + Ti.Utils.base64encode(auth.username + ":" + auth.password);
        }
        Object.keys(headers).forEach(function (name) {
            var value = headers[name];
            if (typeof requestData === 'undefined' && name.toLowerCase() === 'content-type') {
                delete headers[name];
            }
            else {
                client.setRequestHeader(name, value);
            }
        });
        // send request
        client.send(requestData);
    });
};
exports.default = titaniumAdapter;
